module splineMod
    use iso_fortran_env, only: dp=>real64
    implicit none

    contains
    subroutine splie2(x1a,x2a,ya,m,n,y2a)
        integer, intent(in) :: m,n
        real (kind=dp), intent(in) :: x1a(m),x2a(n),ya(n,m)
        real (kind=dp), intent(out):: y2a(n,m)
        integer :: j

        do j=1,m
            call spline(x2a,ya(:,j),n,y2a(:,j))
        enddo
    end subroutine splie2

    subroutine splin2(x1a,x2a,ya,y2a,m,n,x1,x2,y)
        integer, intent(in) :: m,n
        real (kind=dp), intent(in) :: x1a(m),x2a(n),ya(n,m),y2a(n,m)
        real (kind=dp), intent(in) :: x1,x2
        real (kind=dp), intent(out) :: y
        real (kind=dp) :: ytmp(n),yytmp(m)
        integer :: j

        do j=1,m
            call splint(x2a,ya(:,j),y2a(:,j),n,x2,yytmp(j))
        enddo
        call spline(x1a,yytmp,m,ytmp)
        call splint(x1a,yytmp,ytmp,m,x1,y)
    end subroutine splin2


    subroutine spline(x,y,n,y2)

        integer, intent(in) :: n
        real (kind=dp), intent(in) :: x(n),y(n)
        real (kind=dp), intent(out) :: y2(n)
        real (kind=dp) :: u(n),sig,p
        integer :: i

        y2(1)=0.0_dp; y2(n)=0.0d0; u(1)=0.0_dp

        do i=2,n-1
            sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
            p=sig*y2(i-1)+2.0_dp
            y2(i)=(sig-1.0_dp)/p
            u(i)=(6.*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
        enddo

        do i=n-1,1,-1
            y2(i)=y2(i)*y2(i+1)+u(i)
        enddo
    end subroutine spline


    subroutine splint(xa,ya,y2a,n,x,y)
        integer, intent(in) :: n
        real (kind=dp), intent(in) :: xa(n),ya(n),y2a(n),x
        real (kind=dp), intent(out) :: y
        real (kind=dp) :: h,a,b
        integer :: klo,khi,k

        klo=1; khi=n

        do while((khi-klo)>1)
            k=(khi+klo)/2
            if(xa(k)>x)then
                khi=k
            else
                klo=k
            endif
        enddo

        h=xa(khi)-xa(klo)
        if (h==0.) STOP "bad xa input in splint"
        a=(xa(khi)-x)/h
        b=(x-xa(klo))/h
        y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.
    end subroutine splint
end module splineMod


module potMod
    use iso_fortran_env, only: dp=>real64
    use splineMod
    implicit none
    real(kind=dp), parameter :: pi   = 4.0d0*atan(1.0_dp)

    integer, parameter :: nmode = 11, nstate = 6, nnstate = 21, npair = 18
    integer, parameter :: nRho = 50, nPhi = 181

    real(kind=dp), dimension(nPhi,nRho,npair,nnstate) :: pot, diff
    real(kind=dp) :: rho(nRho),phi(nPhi)

    integer,parameter :: vpair(2,npair) = &
            reshape([1,2, 1,3, 1,4, 2,3, 5,1, 5,6, 6,1, 6,2, 6,3, 6,7, 7,1, 7,2, 7,3, 7,4, 8,9, 10,11, 8,7, 10,7], [2,npair])

    private :: pot,diff,rho,phi,vpair

    contains
    subroutine initialize()
        integer :: pr,ns,rh,ph,fUnit
        character(len=20) :: fName

        ! read files
        do pr = 1, npair
            write(fName,'("Diabatic_q"i0"q"i0".dat")') vpair(:,pr)
            open(newunit=fUnit,file=fName,status='old',action='read')
            do rh = 1, nRho
                do ph = 1, nPhi
                    read(fUnit,*)rho(rh),phi(ph),(pot(ph,rh,pr,ns),ns=1,nnstate)  !<- ignoring bad array access
                enddo
                read(fUnit,*)
            enddo
            close(fUnit)
        enddo

        ! calculate diffs
        do ns = 1, nnstate
            do pr = 1, npair
                call splie2(rho, phi, pot(:,:,pr,ns), nRho, nPhi, diff(:,:,pr,ns))
            enddo
        enddo
    end subroutine initialize


    subroutine potential(newgrid,potVal)
        real(kind=dp), intent(in) :: newgrid(nmode)
        real(kind=dp), intent(out) :: potVal(nnstate)
        real(kind=dp) :: q1,q2,rh,ph,res(npair, nnstate)
        integer :: i,j

        !$omp parallel do default(shared) private(i,j,q1,q2,rh,ph) collapse(2)
        do i = 1,nnstate
            do j = 1,npair
                q1 = newgrid(vpair(1,j)); q2 = newgrid(vpair(2,j))
                rh = sqrt(q1**2 + q2**2)
                ph = atan2(q2,q1) + merge(2.0_dp*pi, 0.0_dp, q2<0.0_dp)
                call splin2(rho,phi,pot(:,:,j,i),diff(:,:,j,i),nRho,nPhi,rh,ph,res(j,i))
           enddo
        enddo
        !$omp end parallel do

        potVal = sum(res, dim=1) ! sum along pair
    end subroutine potential
end module
